import pandas as pd
from rltrade import config
from rltrade.data import load_csv
from rltrade.backtests import get_metrics
from rltrade.models import SmartPortfolioLearnerAgent,SmartPortfolioMainAgent

demo = True
time_frame = '1 day'
train_period = ('2012-01-01','2020-08-01') #for training the model
test_period = ('2020-08-02','2021-12-29') #for trading and backtesting

csv_path = 'testdata/ibkr1day.csv'
path = 'models/learner'

ticker_list = ['pall', 'rio', 'spy', 'tlt', 'jbss', 'aapl']
sec_types = ['STK'] * len(ticker_list)
exchanges = ['SMART'] * len(ticker_list)
# tech_indicators = config.STOCK_INDICATORS_LIST
# additional_indicators = config.ADDITIONAL_STOCK_INDICATORS

tech_indicators = [
    "open_2_sma",
    "close_2_tema",
    "tema"
]

additional_indicators = [
    "max_value_price_5",
    "max_value_price_22",
    "max_value_price_66",
    "max_value_price_1year",
    "max_value_price_3year",
    "max_value_price_5year",
    "min_value_price_5",
]

all_indicators = tech_indicators + additional_indicators


for i,indicator in enumerate(all_indicators):

    PPO_PARAMS = {'ent_coef':0.005,
                'learning_rate':0.01,
                'batch_size':252}

    df = load_csv(csv_path) # run download script before this

    print(f"Iteration: {i}")
    save_path = path+"/"+str(i)
    tech_indicator = [indicator] if indicator in tech_indicators else []
    additional_indicator = [indicator] if indicator in additional_indicators else []

    print(tech_indicator,additional_indicator)

    env_kwargs = {
    "initial_amount": 50_828, 
    "ticker_col_name":"tic",
    "filter_threshold":0.25, #between 0.1 to 1, select percentage of top stocks 0.3 means 30% of top stocks
    "target_metrics":['asset','sortino','calamar','skew','kurtosis'], #asset, cagr, sortino, calamar, skew and kurtosis are available options.
    "tech_indicator_list":tech_indicator + additional_indicator, 
    "reward_scaling": 1}

    agent = SmartPortfolioLearnerAgent("ppo",
                        df=df,
                        ticker_list=ticker_list,
                        sec_types=sec_types,
                        exchanges=exchanges,
                        ticker_col_name="tic",
                        tech_indicators=tech_indicator,
                        additional_indicators=additional_indicator,
                        train_period=train_period,
                        test_period=test_period,
                        env_kwargs=env_kwargs,
                        model_kwargs=PPO_PARAMS,
                        tb_log_name='ppo',
                        time_frame=time_frame,
                        demo=demo,
                        epochs=2)

    agent.train_model()
    df_daily_return,df_actions = agent.make_prediction() #testing model on testing period
    
    agent.save_model(save_path) #save the model for trading

##############
# Main model #
##############
df_actions = pd.DataFrame()
for i in range(len(all_indicators)):
    load_path = path + "/" + str(i)
    df = pd.read_csv(load_path+'/action_df.csv')
    df_actions = df_actions.append(df)
df_actions['date'] = pd.to_datetime(df_actions['date'])

MAIN_PPO_PARAMS = {'ent_coef':0.005,
            'learning_rate':0.01,
            'batch_size':252}

main_env_kwargs = {
    "initial_amount": 50_828, 
    "ticker_col_name":"tic",
    "filter_threshold":0.25, #between 0.1 to 1, select percentage of top stocks 0.3 means 30% of top stocks
    "target_metrics":['asset','sortino','calamar','skew','kurtosis'], #asset, cagr, sortino, calamar, skew and kurtosis are available options.
    "reward_scaling": 1}

agent = SmartPortfolioMainAgent("ppo",
                    df=df_actions,
                    ticker_list=ticker_list,
                    sec_types=sec_types,
                    exchanges=exchanges,
                    ticker_col_name="tic",
                    train_period=train_period,
                    test_period=test_period,
                    env_kwargs=main_env_kwargs,
                    model_kwargs=MAIN_PPO_PARAMS,
                    tb_log_name='ppo',
                    time_frame=time_frame,
                    demo=demo,
                    epochs=10)

agent.train_model()
df_daily_return,df_actions = agent.make_prediction() #testing model on testing period
agent.save_model(path+"/main") #save the model for trading
get_metrics(path+"/main")



import pandas as pd

csv_path = 'data/ibkrcontfutgc5mins.csv'
paths = ['data/ibkrcontfutgc5mins1.csv',
    'data/ibkrcontfutgc5mins2.csv',
    'data/ibkrcontfutgc5mins3.csv',
    'data/ibkrcontfutgc5mins4.csv']

df = pd.DataFrame()
for path in paths:
    temp = pd.read_csv(path)
    df = df.append(temp)

df = df.sort_values(by=['tic','date'])
df.to_csv(csv_path,index=False)


from rltrade import config
from rltrade.data import load_csv
from rltrade.backtests import get_metrics
from rltrade.models import SmartDayTradeForexStaticSLTFDynamicVolititySingleAgent


"""
time_frame - last date available
4h  - '2017-12-01'
1h  - '2020-06-19'
30m - '2021-11-18'
15m - '2021-12-03'
5m -  '2021-10-01'
"""

time_frame = "5m"
train_period = ('2021-03-29','2021-09-01')
test_period = ('2021-09-02','2022-01-17')
start_time = "00:00:00"
end_time = "23:55:00" 

csv_path = 'data/ibkrcontfutgc5mins.csv' # path with .csv extension
path = 'models/daytrades/forex-train11-single-2'


ticker_list = ['GC']
sec_types = ['CONTFUT']
exchanges = ['NYMEX'] 
trades_per_stock = [100] # max number per day is 24h/time_frame

tech_indicators = config.STOCK_INDICATORS_LIST # indicators from stockstats
additional_indicators = config.ADDITIONAL_DAYTRADE_INDICATORS

# tech_indicators = [
#     "open_2_sma",
#     "close_2_tema",
#     "tema",
# ]

# additional_indicators = [
#     'max_value_price_10',
#     'max_value_price_50',
#     'max_value_price_100',
#     'max_value_price_500',
#     'max_value_price_1000',
#     'min_value_price_10',
# ]


env_kwargs = {
    "initial_amount": 17_300, #this does not matter as we are making decision for lots and not money.
    "ticker_col_name":"tic",
    "stop_loss":0.01,
    "take_profit":0.03,
    "max_vix_fix":5,
    "min_vix_fix":0.1,
    "vix_fix_lag":50,
    "lot_size":0.01,
    "target_metrics":['asset','unrealized'], #asset, cagr, sortino, calamar, skew and kurtosis are available options.
    "transaction_cost":0.9, #transaction cost per order
    "tech_indicator_list":tech_indicators + additional_indicators, 
    "reward_scaling": 10}

PPO_PARAMS = {'ent_coef':0.005,
            'learning_rate':0.01,
            'batch_size':288}

def train_model():

    df = load_csv(csv_path) # run the download script first

    agent = SmartDayTradeForexStaticSLTFDynamicVolititySingleAgent("ppo",
                        df=df,
                        account=None,
                        time_frame=time_frame,
                        ticker_list=ticker_list,
                        sec_types = sec_types,
                        trades_per_stock=trades_per_stock,
                        exchanges=exchanges,
                        ticker_col_name="tic",
                        tech_indicators=tech_indicators,
                        additional_indicators=additional_indicators,
                        train_period=train_period,
                        test_period=test_period,
                        start_time=start_time,
                        end_time=end_time,
                        env_kwargs=env_kwargs,
                        model_kwargs=PPO_PARAMS,
                        tb_log_name='ppo',
                        epochs=4)

    agent.train_model()
    agent.save_model(path) #save the model for trading

    df_daily_return,df_actions = agent.make_prediction() #testing model on testing period
    get_metrics(path)


train_model()


from rltrade.data import IBKRDownloader

demo = True
time_frame = '5 mins' # options 1 min, 5 mins, 15 mins. 1 hour , 4 hours, 1 day 
start_date = '2021-03-17'
end_date = '2021-05-17'

csv_path = 'data/ibkrcontfutgc5mins4.csv' # path with .csv extension

ticker_list = ['GC']
sec_types = ['CONTFUT']
exchanges = ['NYMEX']

change_name = False # for changing name of ticker else keep False
ticker_list2 = ['GOLD']
sec_types2 = ['-']
exchanges2 = ['-']

ib = IBKRDownloader(start_date = start_date, # first date
                    end_date = end_date, #last date
                    ticker_list = ticker_list,
                    time_frame=time_frame,
                    sec_types=sec_types,
                    exchanges=exchanges,
                    demo=demo)
                    
if time_frame == '1 day':
    df = ib.fetch_daily_data()
else:
    df = ib.fetch_data()

df = df.sort_values(by=['date','tic'])

if change_name:
    for i in range(len(ticker_list)):
        df.loc[df['tic']==ticker_list[i],'tic'] = ticker_list2[i]
        df.loc[df['tic']==ticker_list[i],'sec'] = sec_types2[i]
        df.loc[df['tic']==ticker_list[i],'exchange'] = exchanges2[i]

df.to_csv(csv_path,index=False)